==> splat-0.07.tar.bz2: splat-0.07/splat.pl <==
#!/usr/bin/perl -w

# Copyright 2003; Shaun Guth <l8nite@l8nite.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use warnings;

use Getopt::Long;
use Pod::Usage;
use POSIX qw(strftime);
use Term::ANSIColor;
use File::Basename qw(dirname);
use File::Find;
use File::Spec::Functions qw(splitdir);

my $OPT = {
	'verbose' => 0,
	'summary' => 0,
	'gethelp' => 0,
	'colored' => 0,
	'sortopt' => 0,
	'reverse' => 0,
	'listmod' => 0,
	'package' => undef,
	'logfile' => undef,
};

my %COLORS = (
	'header'      => 'bold blue',
	'date'        => 'bold green',
	'duration'    => 'bold red',
	'counter'     => 'bold green',
	'pkg_normal'  => 'bold blue',
	'pkg_install' => 'cyan',
);

# time format
my $TF = "%a %b %e %H:%M:%S %Y"; 

# load installed packages
my %INSTALLED = ();
find({ 'wanted' => 
	sub {
		return unless /\.ebuild$/;

		my $basedir = dirname($File::Find::name);
		   $basedir =~ s/^\/var\/db\/pkg//;
		my $package = join('/',(splitdir($basedir))[1,2]);

		$INSTALLED{$package} = 1;

	}, 'no_chdir' => 1, },
	'/var/db/pkg',
);

# Main Program

&parse_command_line();

# A request for 'world' package results in showing all builds
$OPT->{'package'} = "" if $OPT->{'package'} eq 'world';

# Colored output?  I think not!
$ENV{'ANSI_COLORS_DISABLED'} = 1 unless $OPT->{'colored'};

my $rawlog = &fetch_portage_log($OPT->{'logfile'});
my $parsed = &parse_portage_log($rawlog);
my $emerge = &fetch_emerge_info($parsed,$OPT->{'package'});

my $summary = {
	'emerges' => 0,
	'duration' => 0,
	'first' => undef,
	'last'   => undef,
};

my $count = keys %$emerge;
if( $count == 0 )
{
	die "!!! Package not found '$OPT->{'package'}'\n";
}

# foreach package that was matched
foreach my $package (sort splat_sort keys %$emerge)
{
	# print out the package name
	unless( $OPT->{'summary'} )
	{
		print $OPT->{'listmod'}
			? colored( "$package\n", &pkgcolor($package) )
			: colored( " * $package\n\n", &pkgcolor($package) );
	}

	# print out the build information for each one
	my @times = @{$emerge->{$package}};
	while ( my ($start,$end) = splice(@times,0,2) )
	{
		my $duration = ($end-$start);

		if( $duration < 0 )
		{
			$duration = 0;
			if( $OPT->{'verbose'} )
			{
				warn "*** Invalid timestamp for $package\n";
			}
		}

		unless( $OPT->{'summary'} || $OPT->{'listmod'} )
		{
			# standard output
			print "\tEmerged at: ",
				colored( strftime($TF, localtime($start)),
					 $COLORS{'date'} ),
				"\n";

			print "\tBuild time: ",
				colored( &duration_exact($duration),
					 $COLORS{'duration'} ),
				"\n";
		}

		# summary information
		$summary->{'emerges'}++;
		$summary->{'duration'} += $duration;

		if( !defined($summary->{'first'}) ||
			$start < $summary->{'first'}->{'start'} )
		{
			$summary->{'first'} = {
				'name'     => $package,
				'start'    => $start,
				'end'      => $end,
			};
		}

		if( !defined($summary->{'last'}) ||
			$end > $summary->{'last'}->{'end'} )
		{
			$summary->{'last'} = {
				'name'     => $package,
				'start'    => $start,
				'end'      => $end,
			};
		}
		# end summary info

		unless( $OPT->{'summary'} || $OPT->{'listmod'} )
		{
			print "\n" if @times;
		}
	}

	unless( $OPT->{'summary'} || $OPT->{'listmod'} )
	{
		print "\n";
	}
}

# output the summary 
if( $OPT->{'summary'} )
{
	print colored( " = Summary\n", $COLORS{'header'} );
	print "\n";
	my $of = "\t%-14s %s\n";

	my $emerges  = $summary->{'emerges'};
	my $duration = $summary->{'duration'};
	my $average  = int($duration/$emerges);

	printf( $of, "Total Builds:",
		colored( $emerges, 
			 $COLORS{'counter'} ) );

	printf( $of, "Time Elapsed:", 
		colored( &duration_exact($duration), 
			 $COLORS{'duration'} ) );

	printf( $of, "Average Time:",
		colored( &duration_exact($average),
			 $COLORS{'duration'} ) );

	print "\n";

	my $first = $summary->{'first'};
	printf( $of, "First Build:", 
		colored( $first->{'name'}, &pkgcolor($first->{'name'}) )
		);
	printf( $of, "", "(" . 
		colored( strftime($TF,localtime($first->{'start'})),
			 $COLORS{'date'} ) . 
		")" );

	print "\n";

	if( $summary->{'emerges'} > 1 )
	{
		my $last  = $summary->{'last'};
		printf( $of, "Last Build:", 
			colored( $last->{'name'}, &pkgcolor($last->{'name'}) )
			);
		printf( $of, "", "(" . 
			colored( strftime($TF,localtime($last->{'end'})),
				 $COLORS{'date'} ) .
			 ")" );

		print "\n";
	}
}

exit;
# end main program, subroutines follow

# gets the color for a package
sub pkgcolor
{
	my $package = shift;

	return exists $INSTALLED{$package}
		? $COLORS{'pkg_install'}
		: $COLORS{'pkg_normal'};
}


# sorts package name keys according to options
sub splat_sort
{
	my $comparison;
	if( $OPT->{'sortopt'} eq 'name' )
	{
		$comparison = $a cmp $b || $a <=> $b;
	}
	elsif( $OPT->{'sortopt'} eq 'date' )
	{
		$comparison = $emerge->{$a}->[0] <=> $emerge->{$b}->[0];
	}
	elsif( $OPT->{'sortopt'} eq 'time' )
	{
		$comparison =   ( $emerge->{$a}->[1] - $emerge->{$a}->[0] )
				<=>
				( $emerge->{$b}->[1] - $emerge->{$b}->[0] );
	}

	$comparison *= -1 if $OPT->{'reverse'};

	return $comparison;
}


# reads the command line arguments using Getopt::Long and sets the appropriate
# values in the global $OPT hash
sub parse_command_line
{
	my $parser = Getopt::Long::Parser->new();
	$parser->configure( 'bundling' );
	$parser->getoptions(
		'verbose|v+' => \$OPT->{'verbose'},
		'summary!'   => \$OPT->{'summary'},
		's!'         => \$OPT->{'summary'},
		'colored!'   => \$OPT->{'colored'},
		'c!'         => \$OPT->{'colored'},
		'help|h|?'   => \$OPT->{'gethelp'},
		'package|p=s'=> \$OPT->{'package'},
		'logfile|f=s'=> \$OPT->{'logfile'},
		'sort|t=s'   => \$OPT->{'sortopt'},
		'reverse|r'  => \$OPT->{'reverse'},
		'list|l'     => \$OPT->{'listmod'},
		) or pod2usage(2);

	pod2usage(1) if $OPT->{'gethelp'};

	if( !defined($OPT->{'package'}) && !defined($ARGV[0]) )
	{
		pod2usage(1);
	}

	$OPT->{'package'} ||= $ARGV[0];
	$OPT->{'sortopt'} ||= 'name';

	unless( $OPT->{'sortopt'} eq 'name' or
		$OPT->{'sortopt'} eq 'time' or
		$OPT->{'sortopt'} eq 'date' )
	{
		die "!!! Cannot sort by '$OPT->{'sortopt'}'\n";
	}
}


# opens the portage log file and gets the contents
# returns an arrayref of lines in the file (chomped)
sub fetch_portage_log
{
	my $logfile = shift;

	# no logfile defined? Let's check make.conf
	if( !defined($logfile) || $logfile eq "" )
	{
		$logfile = '/var/log/emerge.log';

		if( $OPT->{'verbose'} ) {
			warn "*** No log file specified.\n";
			warn "*** Using '$logfile' as log file.\n";
		}
	}

	# logfile doesn't exist?
	unless( -e $logfile )
	{
		die "!!! Fatal: '$logfile' not found\n";
	}

	# load 'em up, move 'em out!
	open( my $fh, '<', $logfile ) 
		or die "!!! Fatal: Unable to open '$logfile'\n";
	my @lines = <$fh>;
	chomp(@lines);
	close( $fh );

	return \@lines;
}


# takes an arrayref of lines that are entries in a portage log file and returns
# an arrayref of only those lines that are part of an actual emerge.  It also
# returns the arrayref in order (start emerge, complete emerge, etc)
sub parse_portage_log
{
	@_ = @{$_[0]};

	# Just get start/finish lines for emerges
	# we're lucky here, they both contain 'emerge (y of y)'
	my @lines = grep ( /emerge \(\d+ of \d+\)/, @_ );
	my @skip  = grep ( !/^\d+:/, @lines );
	   @lines = grep ( /^\d+:/, @lines );

	# We didn't get any lines from our grep? =/
	unless( @lines )
	{
		die "!!! Fatal: Invalid file format.\n";
	}

	# Let them know that we skipped a bit of their logfile
	if( $OPT->{'verbose'} ) {
		my $skip = @skip;
		warn "*** Skipped $skip line(s) due to invalid format.\n";
	}

	# For each 'initiated' emerge, we scan the remaining log for a
	# corresponding 'completed' emerge.  If none is found, then it is an
	# invalid emerge, otherwise we store them.  This sucks because it's
	# slow, but unfortunately portage's logging mechanism screws up if
	# you run two emerges at once, cancel often, etc... 

	my @log;
	while( @lines )
	{
		# skip if we encounter a 'completed' as the first line
		my $line = shift @lines;
		next if $line =~ /completed/;

		my ($timestamp,$eline) = $line =~
			m/^(\d+):  (?:>>>|:::) emerge \(\d+ of \d+\) (.*?)$/;

		my $flag;
		foreach my $x ( 0 .. $#lines )
		{
			# look for a matching line
			if( $lines[$x] =~ m/emerge \(\d+ of \d+\) \Q$eline\E/ )
			{
				# set $flag to indicate this was completed line
				if( $lines[$x] =~ m/completed/ ) {
					$flag = $x; last;
				# or just get out of the inner loop
				} else { last; }
			}
		}

		# was the flag set? if so we'll save the pair of lines
		if( defined($flag) )
		{
			push(@log,$line,splice(@lines,$flag,1));
		}
		# otherwise we continue, since we already shifted the line off
	}

	return \@log;
}


# The log array is expected to be in the format:
# [ 'initiated line', 'completed line', ..., ... ]
# returns a hash in the format:
# {
#   'package' => [ 'ts_start', 'ts_end', ..., ... ],
#   ...
# }
sub fetch_emerge_info
{
	my ($log,$input) = @_;
	my @lines = @{$log};

	# if they typed a category it saves us much despair!
	# (not really, but despair is such a cool word ... )
	my $using_category = $input =~ /^.+\/.+$/;

	my $info;

	while( @lines )
	{
		my $line = shift @lines;
		my ($timestamp,$message) = $line =~ 
			m/^(\d+):\s+(?:>>>|:::)\s+(.*?)$/;

		my ($package) = $message =~
			m/^(?:completed )?emerge \(\d+ of \d+\) (.+?) to /;

		if( $using_category )
		{
			if( $package =~ /^\Q$input\E/ )
			{
				push(@{$info->{$package}},$timestamp);
			}
		}
		else
		{
			my ($category,$application) = $package =~
				m/^(.+?)\/(.+?)$/;

			if( $application =~ /^\Q$input\E/ )
			{
				push(@{$info->{$package}},$timestamp);
			}
		}
	}

	return $info;
}

# Returns a nice human-readable string displaying the duration of the input
# (assuming the input is in seconds)
sub duration_exact
{
	my $ela = shift;
	return 'Invalid timestamp' unless $ela > 0;

	use integer;

	my $day = $ela / 86400; $ela %= 86400;
	my $yea = $day / 365;   $day %= 365;
	my $cen = $yea / 100;   $yea %= 100;
	my $mil = $cen / 10;    $cen %= 10;
	my $hou = $ela / 3600;  $ela %= 3600;
	my $min = $ela / 60;    $ela %= 60;
	my $sec = $ela;

	my $_p = sub
	{
		my ($w,$e1,$e2,$t) = @_;
		return $t != 1 ? "$w$e2" : "$w$e1";
	};

	my @list = ();
	push( @list, "$mil " . &$_p('milleni','um','a',  $mil) ) if $mil;
	push( @list, "$cen " . &$_p('centur', 'y', 'ies',$cen) ) if $cen;
	push( @list, "$yea " . &$_p('year',   '',  's',  $yea) ) if $yea;
	push( @list, "$day " . &$_p('day',    '',  's',  $day) ) if $day;
	push( @list, "$hou " . &$_p('hour',   '',  's',  $hou) ) if $hou;
	push( @list, "$min " . &$_p('minute', '',  's',  $min) ) if $min;
	push( @list, ($min ? "and " : "") .
		     "$sec " . &$_p('second', '',  's',  $sec) ) if $sec;
	my $output = join(', ', @list );


	return $output;
}


__END__

=head1 NAME

splat - Simple Portage Log Analyzing Tool

=head1 SYNOPSIS

splat [options] [--package] package

  Options:
    --help -? -h      brief help message
    --verbose -v      print extra warnings during parsing
    --summary -s      print a summary line instead of full info
    --logfile -f      use specified logfile as emerge log
    --colored -c      output with colors
    --list    -l      output one package name per line

    --package -p      a package name to match. can be a partial package 
                      name (ie 'evo'), in which case it will match any
                      package that starts with 'evo'. you can also 
                      include the category (ie 'net-www/evo')

    --sort    -t      sort the output, accepts a single argument to set
                      which field to sort on: 'date' will sort by the
		      date emerged, 'time' will sort by the time spent 
		      emerging, and 'name' is the default

    --reverse -r      reverse the order of the sorted output

=head1 DESCRIPTION

B<Splat> will attempt to parse your portage log and show information about the
various times you have emerged a particular package (including the build time)

=cut

==> splat-0.08.tar.bz2: splat-0.08/splat.pl <==
#!/usr/bin/perl -w

# Copyright 2003; Shaun Guth <l8nite@l8nite.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use warnings;

use Getopt::Long;
use Pod::Usage;
use POSIX qw(strftime);
use Term::ANSIColor;
use File::Basename qw(dirname);
use File::Find;
use File::Spec::Functions qw(splitdir);

my $OPT = {
	'verbose' => 0,
	'summary' => 0,
	'gethelp' => 0,
	'colored' => 0,
	'sortopt' => 0,
	'reverse' => 0,
	'listmod' => 0,
	'limited' => 0,
	'package' => undef,
	'logfile' => undef,
};

my %COLORS = (
	'header'      => 'bold blue',
	'date'        => 'bold green',
	'duration'    => 'bold red',
	'counter'     => 'bold green',
	'pkg_normal'  => 'bold blue',
	'pkg_install' => 'cyan',
);

# time format
my $TF = "%a %b %e %H:%M:%S %Y"; 

my %INSTALLED = ();
my $emerge;


# Main Program
sub main
{
	# load installed packages
	find({ 'wanted' => 
		sub {
			return unless /\.ebuild$/;

			my $basedir = dirname($File::Find::name);
			   $basedir =~ s/^\/var\/db\/pkg//;
			my $package = join('/',(splitdir($basedir))[1,2]);

			$INSTALLED{$package} = 1;

		}, 'no_chdir' => 1, },
		'/var/db/pkg',
	);

	&parse_command_line();

	# A request for 'world' package results in showing all builds
	$OPT->{'package'} = "" if $OPT->{'package'} eq 'world';

	# Colored output?  I think not!
	$ENV{'ANSI_COLORS_DISABLED'} = 1 unless $OPT->{'colored'};

	my $rawlog = &fetch_portage_log($OPT->{'logfile'});
	my $parsed = &parse_portage_log($rawlog);
	$emerge = &fetch_emerge_info($parsed,$OPT->{'package'});

	my $summary = {
		'emerges' => 0,
		'duration' => 0,
		'first' => undef,
		'last'   => undef,
	};

	my $count = keys %$emerge;
	if( $count == 0 )
	{
		die "!!! Package not found '$OPT->{'package'}'\n";
	}

	# foreach package that was matched
	my $current_idx = 0;
	foreach my $package (sort splat_sort keys %$emerge)
	{
		if( $OPT->{'limited'} && $current_idx++ >= $OPT->{'limited'} )
		{
			last;
		}

		# print out the package name
		unless( $OPT->{'summary'} )
		{
			print $OPT->{'listmod'}
			? colored( "$package\n", &pkgcolor($package) )
			: colored( " * $package\n\n", &pkgcolor($package) );
		}

		# print out the build information for each one
		my @times = @{$emerge->{$package}};
		while ( my ($start,$end) = splice(@times,0,2) )
		{
			my $duration = ($end-$start);

			if( $duration < 0 )
			{
				$duration = 0;
				if( $OPT->{'verbose'} )
				{
				warn "*** Invalid timestamp for $package\n";
				}
			}

			unless( $OPT->{'summary'} || $OPT->{'listmod'} )
			{
				# standard output
				print "\tEmerged at: ",
				colored( strftime($TF, localtime($start)),
					 $COLORS{'date'} ),
				"\n";

				print "\tBuild time: ",
				colored( &duration_exact($duration),
					 $COLORS{'duration'} ),
				"\n";
			}

			# summary information
			$summary->{'emerges'}++;
			$summary->{'duration'} += $duration;

			if( !defined($summary->{'first'}) ||
				$start < $summary->{'first'}->{'start'} )
			{
				$summary->{'first'} = {
					'name'     => $package,
					'start'    => $start,
					'end'      => $end,
				};
			}

			if( !defined($summary->{'last'}) ||
				$end > $summary->{'last'}->{'end'} )
			{
				$summary->{'last'} = {
					'name'     => $package,
					'start'    => $start,
					'end'      => $end,
				};
			}
			# end summary info

			unless( $OPT->{'summary'} || $OPT->{'listmod'} )
			{
				print "\n" if @times;
			}
		}

		unless( $OPT->{'summary'} || $OPT->{'listmod'} )
		{
			print "\n";
		}
	}

	# output the summary 
	if( $OPT->{'summary'} )
	{
		print colored( " = Summary\n", $COLORS{'header'} );
		print "\n";
		my $of = "\t%-14s %s\n";

		my $emerges  = $summary->{'emerges'};
		my $duration = $summary->{'duration'};
		my $average  = int($duration/$emerges);

		printf( $of, "Total Builds:",
			colored( $emerges, 
				 $COLORS{'counter'} ) );

		printf( $of, "Time Elapsed:", 
			colored( &duration_exact($duration), 
				 $COLORS{'duration'} ) );

		printf( $of, "Average Time:",
			colored( &duration_exact($average),
				 $COLORS{'duration'} ) );

		print "\n";

		my $first = $summary->{'first'};
		printf( $of, "First Build:", 
		colored( $first->{'name'}, &pkgcolor($first->{'name'}) )
		);
		printf( $of, "", "(" . 
		colored( strftime($TF,localtime($first->{'start'})),
			 $COLORS{'date'} ) . 
		")" );

		print "\n";

		if( $summary->{'emerges'} > 1 )
		{
			my $last  = $summary->{'last'};
			printf( $of, "Last Build:", 
			colored( $last->{'name'}, &pkgcolor($last->{'name'}) )
			);
			printf( $of, "", "(" . 
			colored( strftime($TF,localtime($last->{'end'})),
				 $COLORS{'date'} ) .
			 ")" );

			print "\n";
		}
	}
}


# gets the color for a package
sub pkgcolor
{
	my $package = shift;

	return exists $INSTALLED{$package}
		? $COLORS{'pkg_install'}
		: $COLORS{'pkg_normal'};
}


# sorts package name keys according to options
sub splat_sort
{
	my $comparison;
	if( $OPT->{'sortopt'} eq 'name' )
	{
		$comparison = $a cmp $b || $a <=> $b;
	}
	elsif( $OPT->{'sortopt'} eq 'date' )
	{
		$comparison = $emerge->{$a}->[0] <=> $emerge->{$b}->[0];
	}
	elsif( $OPT->{'sortopt'} eq 'time' )
	{
		$comparison =   ( $emerge->{$a}->[1] - $emerge->{$a}->[0] )
				<=>
				( $emerge->{$b}->[1] - $emerge->{$b}->[0] );
	}

	$comparison *= -1 if $OPT->{'reverse'};

	return $comparison;
}


# reads the command line arguments using Getopt::Long and sets the appropriate
# values in the global $OPT hash
sub parse_command_line
{
	my $parser = Getopt::Long::Parser->new();
	$parser->configure( 'bundling' );
	$parser->getoptions(
		'verbose|v+' => \$OPT->{'verbose'},
		'summary!'   => \$OPT->{'summary'},
		's!'         => \$OPT->{'summary'},
		'colored!'   => \$OPT->{'colored'},
		'c!'         => \$OPT->{'colored'},
		'help|h|?'   => \$OPT->{'gethelp'},
		'package|p=s'=> \$OPT->{'package'},
		'logfile|f=s'=> \$OPT->{'logfile'},
		'sort|t=s'   => \$OPT->{'sortopt'},
		'reverse|r'  => \$OPT->{'reverse'},
		'list|l'     => \$OPT->{'listmod'},
		'count|u=i'  => \$OPT->{'limited'},
		) or pod2usage(2);

	pod2usage(1) if $OPT->{'gethelp'};

	if( !defined($OPT->{'package'}) && !defined($ARGV[0]) )
	{
		pod2usage(1);
	}

	$OPT->{'package'} ||= $ARGV[0];
	$OPT->{'sortopt'} ||= 'name';

	unless( $OPT->{'sortopt'} eq 'name' or
		$OPT->{'sortopt'} eq 'time' or
		$OPT->{'sortopt'} eq 'date' )
	{
		die "!!! Cannot sort by '$OPT->{'sortopt'}'\n";
	}
}


# opens the portage log file and gets the contents
# returns an arrayref of lines in the file (chomped)
sub fetch_portage_log
{
	my $logfile = shift;

	# no logfile defined? Let's check make.conf
	if( !defined($logfile) || $logfile eq "" )
	{
		$logfile = '/var/log/emerge.log';

		if( $OPT->{'verbose'} ) {
			warn "*** No log file specified.\n";
			warn "*** Using '$logfile' as log file.\n";
		}
	}

	# logfile doesn't exist?
	unless( -e $logfile )
	{
		die "!!! Fatal: '$logfile' not found\n";
	}

	# load 'em up, move 'em out!
	open( my $fh, '<', $logfile ) 
		or die "!!! Fatal: Unable to open '$logfile'\n";
	my @lines = <$fh>;
	chomp(@lines);
	close( $fh );

	return \@lines;
}


# takes an arrayref of lines that are entries in a portage log file and returns
# an arrayref of only those lines that are part of an actual emerge.  It also
# returns the arrayref in order (start emerge, complete emerge, etc)
sub parse_portage_log
{
	@_ = @{$_[0]};

	# Just get start/finish lines for emerges
	# we're lucky here, they both contain 'emerge (y of y)'
	my @lines = grep ( /emerge \(\d+ of \d+\)/, @_ );
	my @skip  = grep ( !/^\d+:/, @lines );
	   @lines = grep ( /^\d+:/, @lines );

	# We didn't get any lines from our grep? =/
	unless( @lines )
	{
		die "!!! Fatal: Invalid file format.\n";
	}

	# Let them know that we skipped a bit of their logfile
	if( $OPT->{'verbose'} ) {
		my $skip = @skip;
		warn "*** Skipped $skip line(s) due to invalid format.\n";
	}

	# For each 'initiated' emerge, we scan the remaining log for a
	# corresponding 'completed' emerge.  If none is found, then it is an
	# invalid emerge, otherwise we store them.  This sucks because it's
	# slow, but unfortunately portage's logging mechanism screws up if
	# you run two emerges at once, cancel often, etc... 

	my @log;
	while( @lines )
	{
		# skip if we encounter a 'completed' as the first line
		my $line = shift @lines;
		next if $line =~ /completed/;

		my ($timestamp,$eline) = $line =~
			m/^(\d+):  (?:>>>|:::) emerge \(\d+ of \d+\) (.*?)$/;

		my $flag;
		foreach my $x ( 0 .. $#lines )
		{
			# look for a matching line
			if( $lines[$x] =~ m/emerge \(\d+ of \d+\) \Q$eline\E/ )
			{
				# set $flag to indicate this was completed line
				if( $lines[$x] =~ m/completed/ ) {
					$flag = $x; last;
				# or just get out of the inner loop
				} else { last; }
			}
		}

		# was the flag set? if so we'll save the pair of lines
		if( defined($flag) )
		{
			push(@log,$line,splice(@lines,$flag,1));
		}
		# otherwise we continue, since we already shifted the line off
	}

	return \@log;
}


# The log array is expected to be in the format:
# [ 'initiated line', 'completed line', ..., ... ]
# returns a hash in the format:
# {
#   'package' => [ 'ts_start', 'ts_end', ..., ... ],
#   ...
# }
sub fetch_emerge_info
{
	my ($log,$input) = @_;
	my @lines = @{$log};

	# if they typed a category it saves us much despair!
	# (not really, but despair is such a cool word ... )
	my $using_category = $input =~ /^.+\/.+$/;

	my $info;

	while( @lines )
	{
		my $line = shift @lines;
		my ($timestamp,$message) = $line =~ 
			m/^(\d+):\s+(?:>>>|:::)\s+(.*?)$/;

		my ($package) = $message =~
			m/^(?:completed )?emerge \(\d+ of \d+\) (.+?) to /;

		if( $using_category )
		{
			if( $package =~ /^\Q$input\E/ )
			{
				push(@{$info->{$package}},$timestamp);
			}
		}
		else
		{
			my ($category,$application) = $package =~
				m/^(.+?)\/(.+?)$/;

			if( $application =~ /^\Q$input\E/ )
			{
				push(@{$info->{$package}},$timestamp);
			}
		}
	}

	return $info;
}

# Returns a nice human-readable string displaying the duration of the input
# (assuming the input is in seconds)
sub duration_exact
{
	my $ela = shift;
	return 'Invalid timestamp' unless $ela > 0;

	use integer;

	my $day = $ela / 86400; $ela %= 86400;
	my $yea = $day / 365;   $day %= 365;
	my $cen = $yea / 100;   $yea %= 100;
	my $mil = $cen / 10;    $cen %= 10;
	my $hou = $ela / 3600;  $ela %= 3600;
	my $min = $ela / 60;    $ela %= 60;
	my $sec = $ela;

	my $_p = sub
	{
		my ($w,$e1,$e2,$t) = @_;
		return $t != 1 ? "$w$e2" : "$w$e1";
	};

	my @list = ();
	push( @list, "$mil " . &$_p('milleni','um','a',  $mil) ) if $mil;
	push( @list, "$cen " . &$_p('centur', 'y', 'ies',$cen) ) if $cen;
	push( @list, "$yea " . &$_p('year',   '',  's',  $yea) ) if $yea;
	push( @list, "$day " . &$_p('day',    '',  's',  $day) ) if $day;
	push( @list, "$hou " . &$_p('hour',   '',  's',  $hou) ) if $hou;
	push( @list, "$min " . &$_p('minute', '',  's',  $min) ) if $min;
	push( @list, ($min ? "and " : "") .
		     "$sec " . &$_p('second', '',  's',  $sec) ) if $sec;
	my $output = join(', ', @list );


	return $output;
}


exit(&main(@ARGV));

__END__

=head1 NAME

splat - Simple Portage Log Analyzing Tool

=head1 SYNOPSIS

splat [options] [--package] package

  Options:
    --help -? -h      brief help message
    --verbose -v      print extra warnings during parsing
    --summary -s      print a summary line instead of full info
    --logfile -f      use specified logfile as emerge log
    --colored -c      output with colors
    --list    -l      output one package name per line
    --count   -u      limit the number of results

    --package -p      a package name to match. can be a partial package 
                      name (ie 'evo'), in which case it will match any
                      package that starts with 'evo'. you can also 
                      include the category (ie 'net-www/evo')

    --sort    -t      sort the output, accepts a single argument to set
                      which field to sort on: 'date' will sort by the
		      date emerged, 'time' will sort by the time spent 
		      emerging, and 'name' is the default

    --reverse -r      reverse the order of the sorted output

=head1 DESCRIPTION

B<Splat> will attempt to parse your portage log and show information about the
various times you have emerged a particular package (including the build time)

=cut
